/* Beispiele aus dem Buch "LDAP fuer Java-Entwickler", 
 * Stefan Zoerner, 4. aktualisierte Auflage 2013
 *  
 * Webseite zum Buch: http://www.ldapjava.de/ 
 * Software & Support Media GmbH / entwickler.press, http://www.entwickler-press.de/
 */
package ldapbuch.kapitel5.ad;

import java.util.Set;
import java.util.TreeSet;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

/**
 * Gruppen eines Benutzers -- (Listing 5_3).
 * 
 * Ermitteln, zu welchen Gruppen ein Benutzer gehrt. Verbindungsdaten werden
 * teilweise von aussen erwartet (jndi.properties), teilweise als
 * Kommandozeilenparameter.
 * 
 * @author Stefan Z&ouml;rner
 */
public class GruppenEinesBenutzers {

  static String FILTER_GROUPS = "(&(objectClass=group)(member={0}))";

  static String FILTER_USERS = "(&(objectClass=user)(sAMAccountName={0}))";

  /**
   * Einstiegspunkt. Als Kommandozeilenparameter werden als erste beiden Werte
   * die Credentials zur Verbindungsaufnahme erwartet, gefolgt von der Kennung
   * (sAMAccountName) des zu untersuchenden Benutzers.
   * 
   * @param args
   *          Kommandozeilenparameter
   */
  public static void main(String[] args) throws NamingException {

    // Authentifizierung
    InitialDirContext ctx = new InitialDirContext();
    ctx.addToEnvironment(Context.SECURITY_AUTHENTICATION,
        "simple");
    ctx.addToEnvironment(Context.SECURITY_PRINCIPAL, args[0]);
    ctx.addToEnvironment(Context.SECURITY_CREDENTIALS, args[1]);

    // DN zum User ermitteln
    String userId = args[2];
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[0]);
    ctls.setReturningObjFlag(true);

    NamingEnumeration<SearchResult> enm = ctx.search("",
        FILTER_USERS, new String[] { userId }, ctls);

    if (!enm.hasMoreElements()) {
      System.err.println("User nicht im Verzeichnis gefunden.");
    } else {
      SearchResult result = enm.next();
      DirContext userCtx = (DirContext) result.getObject();
      String userDN = userCtx.getNameInNamespace();

      // Algorithmus, Schritt 1
      Set<String> memberships = new TreeSet<String>();
      Set<String> newGroups = new TreeSet<String>();
      enm = ctx.search("", FILTER_GROUPS,
          new String[] { userDN }, ctls);
      while (enm.hasMoreElements()) {
        result = enm.next();
        Context obj = (Context) result.getObject();
        String grpDN = obj.getNameInNamespace();
        memberships.add(grpDN);
      }
      newGroups.addAll(memberships);

      // Algorithms, Schritt 2+3
      while (!newGroups.isEmpty()) {
        Set<String> temp = new TreeSet<String>();
        for (String groupDN : newGroups) {
          enm = ctx.search("", FILTER_GROUPS,
              new String[] { groupDN }, ctls);
          while (enm.hasMoreElements()) {
            result = enm.next();
            Context obj = (Context) result.getObject();
            String gdn = obj.getNameInNamespace();
            if (!memberships.contains(gdn)) {
              temp.add(gdn);
              memberships.add(gdn);
            }
          }
        }
        newGroups = temp;
      }

      for (String member : memberships) {
        System.out.println(member);
      }
    }
  }
}